from django import forms
from django.utils.translation import ugettext_lazy as _

from tagging.forms import TagField

from bookmarks.models import BookmarkInstance


class BookmarkInstanceForm(forms.ModelForm):
    url = forms.URLField(
        label="URL",
        widget=forms.TextInput(attrs={"size": 40}),
    )
    description = forms.CharField(
        max_length=100,
        widget=forms.TextInput(attrs={"size": 40}),
    )
    redirect = forms.BooleanField(label="Redirect", required=False)
    tags = TagField(label="Tags", required=False)

    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(BookmarkInstanceForm, self).__init__(*args, **kwargs)
        # hack to order fields
        self.fields.keyOrder = ['url', 'description', 'note', 'tags', 'redirect']

    def clean(self):
        if 'url' not in self.cleaned_data:
            return
        bookmarks = BookmarkInstance.on_site.filter(
            bookmark__url=self.cleaned_data['url'],
            user=self.user,
        )
        if bookmarks.count() > 0:
            raise forms.ValidationError(
                _("You have already bookmarked this link."),
            )
        return self.cleaned_data

    def should_redirect(self):
        if self.cleaned_data["redirect"]:
            return True
        else:
            return False

    def save(self, commit=True):
        url = self.cleaned_data['url']
        self.instance.bookmark = self.instance.get_or_create_bookmark(url)
        return super(BookmarkInstanceForm, self).save(commit)

    class Meta:
        model = BookmarkInstance
        fields = (
            'bookmark',
            'user',
            'saved',
            'description',
            'note',
            'tags',
            'url',
            'redirect',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import datetime
from django.conf import settings
import tagging.fields


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Bookmark',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.URLField(max_length=511)),
                ('description', models.TextField(verbose_name='description')),
                ('note', models.TextField(verbose_name='note', blank=True)),
                ('has_favicon', models.BooleanField(default=False, verbose_name='has favicon')),
                ('favicon_checked', models.DateTimeField(default=datetime.datetime.now, verbose_name='favicon checked')),
                ('added', models.DateTimeField(default=datetime.datetime.now, verbose_name='added')),
                ('adder', models.ForeignKey(related_name='added_bookmarks', verbose_name='adder', to=settings.AUTH_USER_MODEL)),
                ('sites', models.ManyToManyField(to='sites.Site')),
            ],
            options={
                'ordering': ('-added',),
            },
        ),
        migrations.CreateModel(
            name='BookmarkInstance',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('saved', models.DateTimeField(default=datetime.datetime.now, verbose_name='saved')),
                ('description', models.TextField(verbose_name='description')),
                ('note', models.TextField(verbose_name='note', blank=True)),
                ('tags', tagging.fields.TagField(max_length=255, blank=True)),
                ('bookmark', models.ForeignKey(related_name='saved_instances', verbose_name='bookmark', to='bookmarks.Bookmark')),
                ('user', models.ForeignKey(related_name='saved_bookmarks', verbose_name='user', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from datetime import datetime

from django.test import TestCase

from ..forms import BookmarkInstanceForm
from .factories import BookmarkFactory, UserFactory


class TestBookmarkInstanceForm(TestCase):
    def test_save(self):
        bookmark = BookmarkFactory.create()
        user = UserFactory.create()
        url = u'http://www.example.com'
        description = u'A website'
        data = {
            u'url': url,
            u'description': description,
            u'bookmark': bookmark.pk,
            u'user': user.pk,
            u'saved': datetime.now(),
        }

        form = BookmarkInstanceForm(data=data)
        self.assertTrue(form.is_valid(), form.errors)

        instance = form.save()

        self.assertEqual(instance.bookmark.url, url)
        self.assertEqual(instance.description, description)



from unittest import expectedFailure

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.conf import settings

from bookmarks.models import Bookmark, BookmarkInstance
from voting.models import Vote



class UserBookmarksTests(TestCase):
    # FIXME: update the fixtures so they load, or replace with setUp method
    # fixtures = ['accountdata.json', 'bookmarks.json', 'sites.json']

    def checkAddBookmark(self, test_url = 'http://google.com/', test_description = 'test bookmark description', user=None):
        ' Check if new "add bookmark" page works. Can be used to check if new bookmark is added or if existing bookmark is saved. '

        all_bookmarks_url = reverse('all_bookmarks')

        url = reverse('add_bookmark')
        response = self.client.post(url, {'url': test_url, 'description' : test_description, 'tags': 'new-tag'}, follow=True)
        self.assertRedirects(response, all_bookmarks_url)
        self.assertContains(response, 'new-tag')

        bookmark_instance = BookmarkInstance.on_site.order_by('-id')[0]
        bookmark = Bookmark.on_site.all().order_by('-id')[0]

        # Check if it is the new bookmark and new instance is created.
        self.assertEquals(bookmark.url, test_url)
        self.assertEquals(bookmark.sites.all()[0].id, 1)
        self.assertEquals(bookmark.sites.all()[0].id, 1)
        self.assertEquals(bookmark_instance.description, test_description)
        self.assertEquals(bookmark_instance.user, user)
        self.assertEquals(bookmark_instance.bookmark, bookmark)

        # Check if the new bookmark has appeared on the page
        response = self.client.get(all_bookmarks_url)
        self.assertContains(response, test_url, 3, 200,)
        self.assertContains(response, test_description, 1, 200,)

        #Try to add existing bookmark.
        response = self.client.post(url, {'url': test_url, 'description' : test_description,}, follow=True)
        self.assertEquals(response.status_code, 200)
        self.assertFormError(response, 'bookmark_form', None, 'You have already bookmarked this link.', msg_prefix='')

        self.client.logout()


    # Tests if user can rate bookmarks
    def rateBookmark(self, direction, bookmark=None):
        ' Check if a bookmark can be voted correctly.'
        assert direction in ['up', 'down']

        user = User.objects.get(username='testuser')
        self.client.login(username=user.username, password='test')

        if bookmark == None:
            bookmark = Bookmark.on_site.all()[0]

        # Get initial Score value.
        initial = Vote.objects.get_score(bookmark)
        initial_score = int(initial['score'])
        initial_num = int(initial['num_votes'])

        # Vote
        url = reverse('voting.views.vote_on_object', kwargs={'object_id' : bookmark.id, 'direction': direction, }, )
        response = self.client.post(url, **{'HTTP_X_REQUESTED_WITH': 'XMLHttpRequest'})
        self.assertContains(response, '"success": true',  1, 200)

        # Compare with new score value
        new = Vote.objects.get_score(bookmark)
        new_score = int(new['score'])
        new_num = int(new['num_votes'])
        if direction == 'up':
            self.assertEquals(new_score, initial_score+1)
        else:
            self.assertEquals(new_score, initial_score-1)
        self.assertEquals(new_num, initial_num+1)

        # Check if ajax response contains correct values
        self.assertContains(response, '"score": ' + str(new_score), 1, 200)
        self.assertContains(response, '"num_votes": ' + str(new_num), 1, 200)

        self.client.logout()



    # Tests if user can add new bookmark and rate it.
    @expectedFailure
    def testUserBookmarkNew(self):
        ' Tests if a user can add bookmarks correctly. '
        user = User.objects.get(username='testuser')
        self.client.login(username=user.username, password='test')

        url = reverse('add_bookmark')
        response = self.client.get(url,)
        self.assertEquals(response.status_code, 200)

        self.checkAddBookmark(user=user)

        self.client.logout()


    @expectedFailure
    def testVoteUp(self):

        self.rateBookmark('up')

    @expectedFailure
    def testVoteDown(self):

        self.rateBookmark('down')


    @expectedFailure
    def testSaveBookmark(self):
        ' Tests if user can save bookmark. '
        user = User.objects.get(username='testuser')
        self.client.login(username=user.username, password='test')

        bookmark = Bookmark.objects.all()[1]

        url = reverse('add_bookmark')
        response = self.client.post(url, kwargs={'url':bookmark.url, 'description':bookmark.description})
        self.assertEquals(response.status_code, 200)

        self.checkAddBookmark(test_url=bookmark.url, test_description=bookmark.description, user=user)

        self.client.logout()

    @expectedFailure
    def testSiteSpecificBookmark(self):
        ' Tests site specific bookmarks. '
        user = User.objects.get(username='testuser')
        self.client.login(username=user.username, password='test')

        # Deafault SITE_ID=1
        bookmarks = Bookmark.objects.all()
        bookmark_site1 = bookmarks.get(id=1)
        bookmark_site2 = bookmarks.get(id=2)

        url = reverse('all_bookmarks')
        response = self.client.get(url)
        self.assertContains(response, bookmark_site1.url)
        self.assertContains(response, bookmark_site1.description)
        self.assertNotContains(response, bookmark_site2.url)
        self.assertNotContains(response, bookmark_site2.description)

        self.assertContains(response, 'tag-site1')
        self.assertNotContains(response, 'tag-site2')

        settings.SITE_ID=2

        response = self.client.get(url)
        self.assertContains(response, bookmark_site2.url)
        self.assertContains(response, bookmark_site2.description)
        self.assertNotContains(response, bookmark_site1.url)
        self.assertNotContains(response, bookmark_site1.description)

        self.assertContains(response, 'tag-site2')
        self.assertNotContains(response, 'tag-site1')

        settings.SITE_ID=1
        self.client.logout()



from datetime import datetime
import urlparse

from django.db import models
from django.conf import settings
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _

from tagging.fields import TagField
from tagging.models import Tag

"""
A Bookmark is unique to a URL whereas a BookmarkInstance represents a
particular Bookmark saved by a particular person.

This not only enables more than one user to save the same URL as a
bookmark but allows for per-user tagging.
"""

# at the moment Bookmark has some fields that are determined by the
# first person to add the bookmark (the adder) but later we may add
# some notion of voting for the best description and note from
# amongst those in the instances.

# Manager for bookmarks only returns those that belong to current site.
class LiveBookmarkManager(models.Manager):
    def get_queryset(self):
        current_site = Site.objects.get_current()
        return super(LiveBookmarkManager, self).get_queryset().filter(sites=current_site)


class Bookmark(models.Model):
    url = models.URLField(max_length=511)
    description = models.TextField(_('description'))
    note = models.TextField(_('note'), blank=True)

    has_favicon = models.BooleanField(_('has favicon'), default=False)
    favicon_checked = models.DateTimeField(_('favicon checked'), default=datetime.now)

    adder = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="added_bookmarks", verbose_name=_('adder'))
    added = models.DateTimeField(_('added'), default=datetime.now)

    sites = models.ManyToManyField(Site)

    objects = models.Manager()  # The default manager.
    on_site = LiveBookmarkManager()

    def get_favicon_url(self, force=False):
        """
        return the URL of the favicon (if it exists) for the site this
        bookmark is on other return None.

        If force=True, the URL will be calculated even if it doesn't
        exist.
        """
        if self.has_favicon or force:
            base_url = '%s://%s' % urlparse.urlsplit(self.url)[:2]
            favicon_url = urlparse.urljoin(base_url, 'favicon.ico')
            return favicon_url
        return None

    def all_tags(self, min_count=False):
        return Tag.objects.usage_for_queryset(BookmarkInstance.on_site.all(), counts=False, min_count=None, filters={'bookmark': self.id})

    def all_tags_with_counts(self, min_count=False):
        return Tag.objects.usage_for_queryset(
            BookmarkInstance.on_site.filter(bookmark=self.pk),
            counts=True,
            min_count=None,
        )

    def site_slugs(self):
        slugs = []
        for site in self.sites.all():
            slugs.append(site.ghtsite.slug)
        return ' '.join(slugs)

    def __unicode__(self):
        return self.url

    def save(self, force_insert=False, force_update=False, *args, **kwargs):
        super(Bookmark, self).save(force_insert, force_update, *args, **kwargs)
        if not self.sites:
            current_site = Site.objects.get_current()
            self.sites.add(current_site)

    class Meta:
        ordering = ('-added',)


# Manager for bookmark instances only returns those that belong to current site.
class LiveBookmarkInstanceManager(models.Manager):
    def get_queryset(self):
        current_site = Site.objects.get_current()
        return super(LiveBookmarkInstanceManager, self).get_queryset().filter(bookmark__sites=current_site).distinct()


class BookmarkInstance(models.Model):
    bookmark = models.ForeignKey(Bookmark, related_name="saved_instances", verbose_name=_('bookmark'))
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="saved_bookmarks", verbose_name=_('user'))
    saved = models.DateTimeField(_('saved'), default=datetime.now)

    description = models.TextField(_('description'))
    note = models.TextField(_('note'), blank=True)

    tags = TagField()

    objects = models.Manager()  # The default manager.
    on_site = LiveBookmarkInstanceManager()

    def get_or_create_bookmark(self, url):
        try:
            bookmark = Bookmark.on_site.get(url=url)
        except Bookmark.DoesNotExist:
            # has_favicon=False is temporary as the view for adding bookmarks will change it
            bookmark = Bookmark(
                url=url,
                description=self.description,
                note=self.note,
                has_favicon=False,
                adder=self.user,
            )
            bookmark.save()
            bookmark.sites.add(Site.objects.get_current())
        return bookmark

    def delete(self):
        bookmark = self.bookmark
        super(BookmarkInstance, self).delete()
        if bookmark.saved_instances.all().count() == 0:
            bookmark.delete()

    def __unicode__(self):
        return _("%(bookmark)s for %(user)s") % {'bookmark':self.bookmark, 'user':self.user}



from django.contrib.sites.models import Site
from django.test import TestCase
from tagging.models import Tag

from .factories import BookmarkFactory, BookmarkInstanceFactory


class TestBookmark(TestCase):
    def test_all_tags_with_counts(self):
        tag = 'tag'
        bookmark = BookmarkFactory.create()
        bookmark.sites.add(Site.objects.get_current())
        BookmarkInstanceFactory.create(bookmark=bookmark, tags=tag)

        tags = bookmark.all_tags_with_counts()

        tag_instance = Tag.objects.get(name=tag)
        self.assertEqual(tags, [tag_instance])

    def test_all_tags_with_counts_exclude_other_bookmarks(self):
        tag = 'tag'
        other_tag = 'other'
        site = Site.objects.get_current()
        bookmark, other_bookmark = BookmarkFactory.create_batch(2)
        bookmark.sites.add(site)
        other_bookmark.sites.add(site)
        BookmarkInstanceFactory.create(bookmark=bookmark, tags=tag)
        BookmarkInstanceFactory.create(bookmark=other_bookmark, tags=other_tag)

        tags = bookmark.all_tags_with_counts()

        other_tag_instance = Tag.objects.get(name=other_tag)
        self.assertNotIn(other_tag_instance, tags)



import factory
from django.contrib.auth.models import User

from ..models import Bookmark, BookmarkInstance


class UserFactory(factory.DjangoModelFactory):
    username = factory.Sequence('user-{}'.format)

    class Meta:
        model = User


class BookmarkFactory(factory.DjangoModelFactory):
    adder = factory.SubFactory(UserFactory)

    class Meta:
        model = Bookmark


class BookmarkInstanceFactory(factory.DjangoModelFactory):
    bookmark = factory.SubFactory(BookmarkFactory)
    user = factory.SubFactory(UserFactory)

    class Meta:
        model = BookmarkInstance
